/* Algoritmul lui Dijkstra
 * O(m * log n)
 */ 
#include <fstream>
#include <vector>
#include <queue>
using namespace std;

ifstream fin("dijkstra.in");
ofstream fout("dijkstra.out");

using VI  = vector<int>;
using VB  = vector<bool>;
using PI  = pair<int, int>; 
using VP  = vector<PI>;
using VVP = vector<vector<PI>>;

const int Inf = 0x3f3f3f3f;
VVP G;
VI d;    // distantele minime de la sursa la celelalte noduri
int n, m;

struct Pair {
	int node;
	int dist;
	bool operator < (const Pair& p) const
	{
		return dist > p.dist;
	}
};

void ReadGraph();
void Write();
void Dijkstra(int x, VI& d);

int main()
{
	ReadGraph();

	Dijkstra(1, d);
	Write();
}

void Dijkstra(int x, VI& d)   // O(m * log n)
{
	priority_queue<Pair> Q; // coada de prioritati
	int y, dx, w;
	d = VI(n + 1, Inf);
	d[x] = 0;
	Q.push({x, 0});
	
	while (!Q.empty())
	{
		x = Q.top().node;
		dx = Q.top().dist;
		Q.pop();
		
		if (dx > d[x])
			continue;
		for (auto& p : G[x])
		{
			y = p.first; w = p.second;
			if (d[y] > d[x] + w)
			{
				d[y] = d[x] + w;
				Q.push({y, d[y]});
			}
		}
	}
}

void ReadGraph()
{
	fin >> n >> m;
	G = VVP(n + 1);
	
	int x, y, w;       // w - ponderea muchiei
	while (m--)
	{
		fin >> x >> y >> w;
		G[x].push_back({y, w});
	}
}

void Write()
{
	for (int i = 2; i <= n; ++i)
		if (d[i] == Inf)
			fout << 0 << ' ';
		else
			fout << d[i] << ' ';	
}
